@@ -31,8 +31,8 @@
 /* Adapted from CVS version 1.10 of glibc's sysdeps/i386/i486/bits/atomic.h 
  */
 gint
-g_atomic_int_exchange_and_add (gint *atomic, 
-			       gint val)
+g_atomic_int_exchange_and_add (volatile gint *atomic, 
+			       gint           val)
 {
   gint result;
 
@@ -43,8 +43,8 @@ g_atomic_int_exchange_and_add (gint *atomic,
 }
  
 void
-g_atomic_int_add (gint *atomic, 
-		  gint val)
+g_atomic_int_add (volatile gint *atomic, 
+		  gint           val)
 {
   __asm__ __volatile__ ("lock; addl %1,%0"
 			: "=m" (*atomic) 
@@ -52,9 +52,9 @@ g_atomic_int_add (gint *atomic,
 }
 
 gboolean
-g_atomic_int_compare_and_exchange (gint *atomic, 
-				   gint oldval, 
-				   gint newval)
+g_atomic_int_compare_and_exchange (volatile gint *atomic, 
+				   gint           oldval, 
+				   gint           newval)
 {
   gint result;
  
@@ -70,9 +70,9 @@ g_atomic_int_compare_and_exchange (gint *atomic,
  * arguments and calling the former function */
 
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gpointer result;
  
@@ -98,9 +98,9 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 
 #  if GLIB_SIZEOF_VOID_P == 4 /* 32-bit system */
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gpointer result;
   __asm__ __volatile__ ("cas [%4], %2, %0"
@@ -111,9 +111,9 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 }
 #  elif GLIB_SIZEOF_VOID_P == 8 /* 64-bit system */
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gpointer result;
   gpointer *a = atomic;
@@ -157,9 +157,9 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
   })
 #  if GLIB_SIZEOF_VOID_P == 4 /* 32-bit system */
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gint result;
   gpointer prev;
@@ -183,9 +183,9 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 }
 #  elif GLIB_SIZEOF_VOID_P == 8 /* 64-bit system */
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gint result;
   gpointer prev;
@@ -215,8 +215,8 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 /* Adapted from CVS version 1.9 of glibc's sysdeps/x86_64/bits/atomic.h 
  */
 gint
-g_atomic_int_exchange_and_add (gint *atomic, 
-			       gint val)
+g_atomic_int_exchange_and_add (volatile gint *atomic,
+			       gint           val)
 {
   gint result;
 
@@ -227,8 +227,8 @@ g_atomic_int_exchange_and_add (gint *atomic,
 }
  
 void
-g_atomic_int_add (gint *atomic, 
-		  gint val)
+g_atomic_int_add (volatile gint *atomic, 
+		  gint           val)
 {
   __asm__ __volatile__ ("lock; addl %1,%0"
 			: "=m" (*atomic) 
@@ -236,9 +236,9 @@ g_atomic_int_add (gint *atomic,
 }
 
 gboolean
-g_atomic_int_compare_and_exchange (gint *atomic, 
-				   gint oldval, 
-				   gint newval)
+g_atomic_int_compare_and_exchange (volatile gint *atomic, 
+				   gint           oldval, 
+				   gint           newval)
 {
   gint result;
  
@@ -250,9 +250,9 @@ g_atomic_int_compare_and_exchange (gint *atomic,
 }
 
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gpointer result;
  
@@ -272,8 +272,8 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 /* Non-optimizing compile bails on the following two asm statements
  * for reasons unknown to the author */
 gint
-g_atomic_int_exchange_and_add (gint *atomic, 
-			       gint val)
+g_atomic_int_exchange_and_add (volatile gint *atomic, 
+			       gint           val)
 {
   gint result, temp;
   __asm__ __volatile__ ("1:       lwarx   %0,0,%3\n"
@@ -288,8 +288,8 @@ g_atomic_int_exchange_and_add (gint *atomic,
  
 /* The same as above, to save a function call repeated here */
 void
-g_atomic_int_add (gint *atomic, 
-		  gint val)
+g_atomic_int_add (volatile gint *atomic, 
+		  gint           val)
 {
   gint result, temp;  
   __asm__ __volatile__ ("1:       lwarx   %0,0,%3\n"
@@ -302,8 +302,8 @@ g_atomic_int_add (gint *atomic,
 }
 #   else /* !__OPTIMIZE__ */
 gint
-g_atomic_int_exchange_and_add (gint *atomic, 
-			       gint val)
+g_atomic_int_exchange_and_add (volatile gint *atomic, 
+			       gint           val)
 {
   gint result;
   do
@@ -314,8 +314,8 @@ g_atomic_int_exchange_and_add (gint *atomic,
 }
  
 void
-g_atomic_int_add (gint *atomic, 
-		  gint val)
+g_atomic_int_add (volatile gint *atomic,
+		  gint           val)
 {
   gint result;
   do
@@ -326,9 +326,9 @@ g_atomic_int_add (gint *atomic,
 
 #   if GLIB_SIZEOF_VOID_P == 4 /* 32-bit system */
 gboolean
-g_atomic_int_compare_and_exchange (gint *atomic, 
-				   gint oldval, 
-				   gint newval)
+g_atomic_int_compare_and_exchange (volatile gint *atomic, 
+				   gint           oldval, 
+				   gint           newval)
 {
   gint result;
   __asm__ __volatile__ ("sync\n"
@@ -345,9 +345,9 @@ g_atomic_int_compare_and_exchange (gint *atomic,
 }
 
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gpointer result;
   __asm__ __volatile__ ("sync\n"
@@ -364,9 +364,9 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 }
 #   elif GLIB_SIZEOF_VOID_P == 8 /* 64-bit system */
 gboolean
-g_atomic_int_compare_and_exchange (gint *atomic, 
-				   gint oldval, 
-				   gint newval)
+g_atomic_int_compare_and_exchange (volatile gint *atomic,
+				   gint           oldval, 
+				   gint           newval)
 {
   gpointer result;
   __asm__ __volatile__ ("sync\n"
@@ -384,9 +384,9 @@ g_atomic_int_compare_and_exchange (gint *atomic,
 }
 
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gpointer result;
   __asm__ __volatile__ ("sync\n"
@@ -411,34 +411,34 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 /* Adapted from CVS version 1.8 of glibc's sysdeps/ia64/bits/atomic.h
  */
 gint
-g_atomic_int_exchange_and_add (gint *atomic, 
-			       gint val)
+g_atomic_int_exchange_and_add (volatile gint *atomic,
+			       gint           val)
 {
-  return __sync_fetch_and_add_si (atomic, val);
+  return __sync_fetch_and_add (atomic, val);
 }
  
 void
-g_atomic_int_add (gint *atomic, 
+g_atomic_int_add (volatile gint *atomic, 
 		  gint val)
 {
-  __sync_fetch_and_add_si (atomic, val);
+  __sync_fetch_and_add (atomic, val);
 }
 
 gboolean
-g_atomic_int_compare_and_exchange (gint *atomic, 
-				   gint oldval, 
-				   gint newval)
+g_atomic_int_compare_and_exchange (volatile gint *atomic,
+				   gint           oldval, 
+				   gint           newval)
 {
-  return __sync_bool_compare_and_swap_si (atomic, oldval, newval);
+  return __sync_bool_compare_and_swap (atomic, oldval, newval);
 }
 
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic,
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
-  return __sync_bool_compare_and_swap_di ((long *)atomic, 
-					  (long)oldval, (long)newval);
+  return __sync_bool_compare_and_swap ((long *)atomic, 
+				       (long)oldval, (long)newval);
 }
 
 #  define G_ATOMIC_MEMORY_BARRIER __sync_synchronize ()
@@ -456,9 +456,9 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 
 #  if GLIB_SIZEOF_VOID_P == 4 /* 32-bit system */
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic,
+				       gpointer           oldval,
+				       gpointer           newval)
 {
   gpointer result = oldval;
   __asm__ __volatile__ ("cs %0, %2, %1"
@@ -468,9 +468,9 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 }
 #  elif GLIB_SIZEOF_VOID_P == 8 /* 64-bit system */
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic,
+				       gpointer           oldval,
+				       gpointer           newval)
 {
   gpointer result = oldval;
   gpointer *a = atomic;
@@ -496,33 +496,33 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 #ifdef DEFINE_WITH_WIN32_INTERLOCKED
 # include <windows.h>
 gint32   
-g_atomic_int_exchange_and_add (gint32   *atomic, 
-			       gint32    val)
+g_atomic_int_exchange_and_add (volatile gint32 *atomic,
+			       gint32           val)
 {
   return InterlockedExchangeAdd (atomic, val);
 }
 
 void     
-g_atomic_int_add (gint32   *atomic, 
-		  gint32    val)
+g_atomic_int_add (volatile gint32 *atomic, 
+		  gint32           val)
 {
   InterlockedExchangeAdd (atomic, val);
 }
 
 gboolean 
-g_atomic_int_compare_and_exchange (gint32   *atomic, 
-				   gint32    oldval, 
-				   gint32    newval)
+g_atomic_int_compare_and_exchange (volatile gint32 *atomic,
+				   gint32           oldval,
+				   gint32           newval)
 {
-  return (guint32)InterlockedCompareExchange ((PVOID*)atomic, 
-                                              (PVOID)newval, 
-                                              (PVOID)oldval) == oldval;
+  return (guint32) InterlockedCompareExchange ((PVOID*)atomic, 
+                                               (PVOID)newval, 
+                                               (PVOID)oldval) == oldval;
 }
 
 gboolean 
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic,
+				       gpointer           oldval,
+				       gpointer           newval)
 {
 # if GLIB_SIZEOF_VOID_P != 4 /* no 32-bit system */
 #  error "InterlockedCompareExchangePointer needed"
@@ -537,8 +537,8 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 static GMutex *g_atomic_mutex; 
 
 gint
-g_atomic_int_exchange_and_add (gint *atomic, 
-			       gint  val)
+g_atomic_int_exchange_and_add (volatile gint *atomic, 
+			       gint           val)
 {
   gint result;
     
@@ -552,8 +552,8 @@ g_atomic_int_exchange_and_add (gint *atomic,
 
 
 void
-g_atomic_int_add (gint *atomic,
-		  gint  val)
+g_atomic_int_add (volatile gint *atomic,
+		  gint           val)
 {
   g_mutex_lock (g_atomic_mutex);
   *atomic += val;
@@ -561,9 +561,9 @@ g_atomic_int_add (gint *atomic,
 }
 
 gboolean
-g_atomic_int_compare_and_exchange (gint *atomic, 
-				   gint  oldval, 
-				   gint  newval)
+g_atomic_int_compare_and_exchange (volatile gint *atomic, 
+				   gint           oldval, 
+				   gint           newval)
 {
   gboolean result;
     
@@ -581,9 +581,9 @@ g_atomic_int_compare_and_exchange (gint *atomic,
 }
 
 gboolean
-g_atomic_pointer_compare_and_exchange (gpointer *atomic, 
-				       gpointer  oldval, 
-				       gpointer  newval)
+g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic, 
+				       gpointer           oldval, 
+				       gpointer           newval)
 {
   gboolean result;
     
@@ -602,7 +602,7 @@ g_atomic_pointer_compare_and_exchange (gpointer *atomic,
 
 #ifdef G_ATOMIC_OP_MEMORY_BARRIER_NEEDED
 gint
-g_atomic_int_get (gint *atomic)
+g_atomic_int_get (volatile gint *atomic)
 {
   gint result;
 
@@ -614,7 +614,7 @@ g_atomic_int_get (gint *atomic)
 }
 
 gpointer
-g_atomic_pointer_get (gpointer *atomic)
+g_atomic_pointer_get (volatile gpointer *atomic)
 {
   gpointer result;
 
@@ -627,32 +627,38 @@ g_atomic_pointer_get (gpointer *atomic)
 #endif /* G_ATOMIC_OP_MEMORY_BARRIER_NEEDED */   
 #elif defined (G_ATOMIC_OP_MEMORY_BARRIER_NEEDED)
 gint
-g_atomic_int_get (gint *atomic)
+g_atomic_int_get (volatile gint *atomic)
 {
+  gint result = *atomic;
+
   G_ATOMIC_MEMORY_BARRIER;
-  return *atomic;
+
+  return result;
 }
 
 gpointer
-g_atomic_pointer_get (gpointer *atomic)
+g_atomic_pointer_get (volatile gpointer *atomic)
 {
+  gpointer result = *atomic;
+
   G_ATOMIC_MEMORY_BARRIER;
-  return *atomic;
+
+  return result;
 }   
 #endif /* DEFINE_WITH_MUTEXES || G_ATOMIC_OP_MEMORY_BARRIER_NEEDED */
 
 #ifdef ATOMIC_INT_CMP_XCHG
 gboolean
-g_atomic_int_compare_and_exchange (gint *atomic, 
-				   gint oldval, 
-				   gint newval)
+g_atomic_int_compare_and_exchange (volatile gint *atomic,
+				   gint           oldval,
+				   gint           newval)
 {
   return ATOMIC_INT_CMP_XCHG (atomic, oldval, newval);
 }
 
 gint
-g_atomic_int_exchange_and_add (gint *atomic, 
-			       gint val)
+g_atomic_int_exchange_and_add (volatile gint *atomic,
+			       gint           val)
 {
   gint result;
   do
@@ -663,8 +669,8 @@ g_atomic_int_exchange_and_add (gint *atomic,
 }
  
 void
-g_atomic_int_add (gint *atomic, 
-		  gint val)
+g_atomic_int_add (volatile gint *atomic,
+		  gint           val)
 {
   gint result;
   do
@@ -683,13 +689,13 @@ _g_atomic_thread_init (void)
 
 #ifndef G_ATOMIC_OP_MEMORY_BARRIER_NEEDED
 gint
-(g_atomic_int_get) (gint *atomic)
+(g_atomic_int_get) (volatile gint *atomic)
 {
   return g_atomic_int_get (atomic);
 }
 
 gpointer
-(g_atomic_pointer_get) (gpointer *atomic)
+(g_atomic_pointer_get) (volatile gpointer *atomic)
 {
   return g_atomic_pointer_get (atomic);
 }
