@@ -370,10 +370,6 @@ extern void glib_dummy_decl (void);
 #  define G_TRYLOCK(name)               (TRUE)
 #endif  /* !G_THREADS_ENABLED */
 
-/* --- internal API --- */
-gpointer _g_thread_mem_private_get (GThread *thread);
-void     _g_thread_mem_private_set (GThread *thread,
-				    gpointer data);
 
 G_END_DECLS
 
